import * as React from 'react';
import { Enter } from '@fluentui/keyboard-keys';
import { Button, mergeCallbacks, Persona, slot, Tooltip, useFluent, useId, useMergedRefs } from '@fluentui/react-components';
import { useInputListboxContext } from '@fluentui-copilot/react-input-listbox';
import { useDesignVersion } from '@fluentui-copilot/react-provider';
import { bundleIcon, ChevronLeftFilled, ChevronLeftRegular, ChevronRightFilled, ChevronRightRegular } from '@fluentui/react-icons';
const ChevronRightIcon = bundleIcon(ChevronRightFilled, ChevronRightRegular);
const ChevronLeftIcon = bundleIcon(ChevronLeftFilled, ChevronLeftRegular);
/**
 * Create the state required to render GroundingMenuItem.
 *
 * The returned state can be modified with hooks such as useGroundingMenuItemStyles_unstable,
 * before being passed to renderGroundingMenuItem_unstable.
 *
 * @param props - props from this instance of GroundingMenuItem
 * @param ref - reference to root HTMLElement of GroundingMenuItem
 */
export const useGroundingMenuItem_unstable = (props, ref) => {
  const {
    className,
    style,
    ...primarySlotProps
  } = props;
  const {
    disabled,
    value,
    onOptionSelect
  } = primarySlotProps;
  const designVersion = useDesignVersion(props.designVersion);
  const {
    dir
  } = useFluent();
  const optionRef = React.useRef(null);
  const rootRef = useMergedRefs(optionRef, ref);
  // use the id if provided, otherwise use a generated id
  const id = useId('grounding-menu-item', props.id);
  const optionData = React.useRef(undefined);
  const selectOption = useInputListboxContext(ctx => ctx.selectOption);
  const selectItem = React.useCallback(event => {
    if (disabled) {
      event.preventDefault();
      return;
    }
    if (optionData.current) {
      selectOption(event, optionData.current);
    }
  }, [disabled, selectOption]);
  const onKeyDown = React.useCallback(event => {
    if (event.key === Enter) {
      selectItem(event);
    }
  }, [selectItem]);
  // Resolve the slot early to ensure we don't overwrite a user provided ref
  const primaryText = slot.optional(props.primaryText, {
    elementType: 'span'
  });
  const textRef = React.useRef();
  const primaryTextRef = useMergedRefs(primaryText === null || primaryText === void 0 ? void 0 : primaryText.ref, textRef);
  if (primaryText) {
    primaryText.ref = primaryTextRef;
  }
  // register option data with context
  const registerOption = useInputListboxContext(ctx => ctx.registerOption);
  React.useEffect(() => {
    if (id && optionRef.current && textRef.current) {
      var _textRef_current_textContent;
      const optionText = (_textRef_current_textContent = textRef.current.textContent) !== null && _textRef_current_textContent !== void 0 ? _textRef_current_textContent : '';
      const optionValue = value !== null && value !== void 0 ? value : optionText;
      // Save a reference to the option data so other callbacks can access it
      optionData.current = {
        id,
        disabled,
        text: optionText,
        value: optionValue,
        onOptionSelect
      };
      const unregisterOption = registerOption(optionData.current, optionRef.current);
      return () => {
        unregisterOption();
        optionData.current = undefined;
      };
    }
  }, [disabled, id, onOptionSelect, registerOption, value]);
  const selected = useInputListboxContext(ctx => {
    var _optionData_current;
    const selectedOptions = ctx.selectedOptions;
    return ((_optionData_current = optionData.current) === null || _optionData_current === void 0 ? void 0 : _optionData_current.value) !== undefined && selectedOptions.find(o => {
      var _optionData_current;
      return o === ((_optionData_current = optionData.current) === null || _optionData_current === void 0 ? void 0 : _optionData_current.value);
    }) !== undefined;
  });
  const state = {
    designVersion,
    disabled,
    selected,
    components: {
      root: 'div',
      persona: Persona,
      submenuButton: Button,
      submenuButtonTooltip: Tooltip
    },
    root: slot.always(props.root, {
      defaultProps: {
        className,
        style
      },
      elementType: 'div'
    }),
    persona: slot.always(props.persona, {
      defaultProps: {
        'aria-disabled': disabled ? true : undefined,
        role: 'menuitem',
        id,
        tabIndex: 0,
        ...primarySlotProps,
        primaryText,
        onKeyDown: mergeCallbacks(onKeyDown, props.onKeyDown),
        onClick: mergeCallbacks(selectItem, props.onClick),
        ref: rootRef
      },
      elementType: Persona
    }),
    submenuButton: slot.optional(props.submenuButton, {
      defaultProps: {
        appearance: 'transparent',
        'aria-haspopup': 'menu',
        role: 'menuitem',
        icon: dir === 'ltr' ? /*#__PURE__*/React.createElement(ChevronRightIcon, null) : /*#__PURE__*/React.createElement(ChevronLeftIcon, null)
      },
      elementType: Button
    }),
    submenuButtonTooltip: slot.optional(props.submenuButtonTooltip, {
      defaultProps: {
        relationship: 'label'
      },
      elementType: Tooltip
    })
  };
  if (state.submenuButton) {
    var _state_root_role;
    state.root.role = (_state_root_role = state.root.role) !== null && _state_root_role !== void 0 ? _state_root_role : 'group';
    state.submenuButton.onClick = mergeCallbacks(state.submenuButton.onClick, ev => {
      ev.stopPropagation();
    });
    state.submenuButton.onKeyDown = mergeCallbacks(state.submenuButton.onKeyDown, ev => {
      if (ev.key === Enter) {
        ev.stopPropagation();
      }
    });
  }
  return state;
};
//# sourceMappingURL=useGroundingMenuItem.js.map